@@ -87,13 +87,22 @@ void BamRecord::assign(
     r_->core.bin = hts_reg2bin(r_->core.pos, r_->core.pos + bam_cigar2rlen(r_->core.n_cigar, cigar), 14, 5);
 }
 
-BamHeader::BamHeader(const char* text, size_t len) {
+BamHeader::BamHeader(const char* text, size_t len): h_() {
     h_ = sam_hdr_parse(len+1, text); // null-terminated
     if (h_ == NULL)
         throw ios::failure("sam_hdr_parse");
     h_->l_text = len+1;
     h_->text = (char*) malloc(h_->l_text);
     strcpy(h_->text, text);
+    init_sdict();
+}
+
+void BamHeader::init_sdict() {
+    if(h_->n_targets == 0)
+        DOES_NOT_HAPPEN;
+    int32_t i = bam_name2id(h_, h_->target_name[0]);
+    assert(h_->sdict);
+    assert(i == 0);
 }
 
 void BamHeader::check_same_ref_chroms(
