@@ -545,13 +545,13 @@ LocPopSum::tally_metapop(const CSLocus *cloc)
 
         if (p_cnt == 1 && q_cnt > 1) {
             for (uint j = 0; j < this->_pop_cnt; j++)
-                if (s[j]->nucs[col].p_nuc() == mp->nucs[col].p_allele ||
-                    s[j]->nucs[col].q_nuc() == mp->nucs[col].p_allele)
+                if ((s[j]->nucs[col].p_nuc() != 0 && s[j]->nucs[col].p_nuc() == mp->nucs[col].p_allele) ||
+                    (s[j]->nucs[col].q_nuc() != 0 && s[j]->nucs[col].q_nuc() == mp->nucs[col].p_allele))
                     variable_pop = j;
         } else if (p_cnt > 1 && q_cnt == 1) {
             for (uint j = 0; j < this->_pop_cnt; j++)
-                if (s[j]->nucs[col].p_nuc() == mp->nucs[col].q_allele ||
-                    s[j]->nucs[col].q_nuc() == mp->nucs[col].q_allele)
+                if ((s[j]->nucs[col].p_nuc() != 0 && s[j]->nucs[col].p_nuc() == mp->nucs[col].q_allele) ||
+                    (s[j]->nucs[col].q_nuc() != 0 && s[j]->nucs[col].q_nuc() == mp->nucs[col].q_allele))
                     variable_pop = j;
         } else if (p_cnt == 1 && q_cnt == 1) {
 	    //
