@@ -144,7 +144,7 @@ inline
 void Seq::reserve(size_t n, bool with_qual) {
     if (capacity < long(n)) {
         capacity = n;
-        delete seq;
+        delete [] seq;
         seq = new char[capacity + 1];
         *seq = '\0';
         if (with_qual) {
