@@ -466,11 +466,13 @@ check_quality_scores(RawRead *href, int qual_offset, int score_limit, int len_li
     }
     // cerr << "\n";
 
+    const int *p, *q;
     double mean        = 0.0;
     double working_sum = 0.0;
-    int *p, *q, j;
+    int j;
 
     // cerr << "Window length: " << href->win_len << "; Stop position: " << href->stop_pos << "\n";
+    
     //
     // Populate the sliding window.
     //
@@ -487,7 +489,7 @@ check_quality_scores(RawRead *href, int qual_offset, int score_limit, int len_li
     j = offset;
 
     // cerr << "Setting pointers; P: " << (href->int_scores + offset) - href->int_scores << "; Q: " << p + (int) href->win_len - p << "; J: " << j << "\n";
-
+    
     do {
         mean = working_sum / href->win_len;
 
