@@ -159,6 +159,12 @@ const unsigned int max_samples = max_alleles / 2;
 //
 const double FSNC = -7.0;
 
+//
+// A flag to store the special case when we are searching for private alleles
+// and find two, alternative private alleles at the same locus.
+//
+const int16_t two_priv_alleles = -2;
+
 //
 // Supported file types
 //
