@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2010-2021, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2010-2023, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -42,6 +42,7 @@ double  min_match_len     = 0.80;
 double  max_gaps          = 2.0;
 
 size_t  next_catalog_id   = 1;
+size_t  next_sample_id    = 1;
 
 using namespace std;
 
@@ -71,14 +72,13 @@ int main (int argc, char* argv[]) {
 
     map<int, CLocus *> catalog;
     pair<int, string>  s;
-    bool compressed = false;
-    int  i;
+    int i;
 
-    set<int> seen_sample_ids; // For checking sample ID unicity.
+    map<string, size_t> seen_samples; // For checking sample ID unicity.
 
     if (catalog_path.length() > 0) {
         cerr << "\nInitializing existing catalog...\n";
-        if (!initialize_existing_catalog(catalog_path, catalog)) {
+        if (!initialize_existing_catalog(catalog_path, catalog, seen_samples, next_sample_id)) {
             cerr << "Error: Failed to initialize the catalog.\n";
             throw exception();
         }
@@ -89,12 +89,11 @@ int main (int argc, char* argv[]) {
         samples.pop();
 
         cerr << "\nInitializing new catalog...\n";
-        if (!initialize_new_catalog(s, catalog)) {
+        if (!initialize_new_catalog(s, catalog, seen_samples, next_sample_id)) {
             cerr << "Error: Failed to initialize the catalog.\n";
             throw exception();
         }
         i = 2;
-        seen_sample_ids.insert(catalog.begin()->second->sample_id);
     }
 
     while (!samples.empty()) {
@@ -105,6 +104,7 @@ int main (int argc, char* argv[]) {
 
         cerr << "\nProcessing sample " << s.second << " [" << i << " of " << sample_cnt << "]\n";
 
+	bool compressed = false;
         if (!load_loci(s.second, sample, 0, false, compressed)) {
             cerr << "Failed to load sample " << i << "\n";
             continue;
@@ -112,15 +112,18 @@ int main (int argc, char* argv[]) {
         //
         // Assign the ID for this sample data.
         //
-        s.first = sample.begin()->second->sample_id;
+        s.first = next_sample_id;
 
-        // Check for unique sample IDs.
-        if (!seen_sample_ids.insert(s.first).second) {
+        // Check for unique sample names.
+        if (seen_samples.count(s.second) > 0) {
             // Insert failed.
-            cerr << "Error: Sample ID '" << s.first << "' occurs more than once. Sample IDs must be unique." << endl;
+            cerr << "Error: Sample '" << s.second << "' is already inserted into this catalog. Samples must be unique." << endl;
             throw exception();
-        }
-
+	} else {
+	    seen_samples[s.second] = next_sample_id;
+	}
+	next_sample_id++;
+	
         //dump_loci(sample);
 
         cerr << "Searching for sequence matches...\n";
@@ -163,7 +166,7 @@ int main (int argc, char* argv[]) {
     cerr << "\nWriting catalog in directory '" << out_path << "'.\n"
          << "Final catalog contains " << catalog.size() << " loci.\n";
 
-    write_catalog(catalog);
+    write_catalog(catalog, seen_samples);
     cerr << "cstacks is done.\n";
 
     //
@@ -177,27 +180,6 @@ int main (int argc, char* argv[]) {
     IF_NDEBUG_CATCH_ALL_EXCEPTIONS
 }
 
-int update_catalog_index(map<int, CLocus *> &catalog, map<string, int> &cat_index) {
-    map<int, CLocus *>::iterator j;
-    char id[id_len];
-
-    for (j = catalog.begin(); j != catalog.end(); j++) {
-        snprintf(id, id_len - 1, "%s|%d|%c",
-                 j->second->loc.chr(),
-                 j->second->loc.bp,
-                 j->second->loc.strand == strand_plus ? '+' : '-');
-
-        if (cat_index.count(id) == 0) {
-            cat_index[id] = j->first;
-        } else {
-            if (cat_index[id] != j->first)
-                cerr << "Error: Catalog index mismatch, key: '" << id << "'.\n";
-        }
-    }
-
-    return 0;
-}
-
 int
 characterize_mismatch_snps(CLocus *catalog_tag, QLocus *query_tag)
 {
@@ -1042,8 +1024,9 @@ int find_matches_by_genomic_loc(map<string, int> &cat_index, map<int, QLocus *>
     return 0;
 }
 
-int write_catalog(map<int, CLocus *> &catalog) {
-
+int
+write_catalog(map<int, CLocus *> &catalog, map<string, size_t> &seen_samples)
+{
     map<int, CLocus *>::iterator i;
     CLocus  *tag;
     set<int> matches;
@@ -1053,18 +1036,24 @@ int write_catalog(map<int, CLocus *> &catalog) {
     string tag_file = out_path + "catalog.tags.tsv";
     string snp_file = out_path + "catalog.snps.tsv";
     string all_file = out_path + "catalog.alleles.tsv";
-
+    string sam_file = out_path + "catalog.sample_list.tsv";
+    
     if (gzip) {
         tag_file += ".gz";
         snp_file += ".gz";
         all_file += ".gz";
+	sam_file += ".gz";
     }
 
+    //
+    // Write the sample list.
+    //
+    
     //
     // Open the output files for writing.
     //
-    gzFile   gz_tags, gz_snps, gz_alle;
-    ofstream tags, snps, alle;
+    gzFile   gz_tags, gz_snps, gz_alle, gz_sams;
+    ofstream tags, snps, alle, sams;
     if (gzip) {
         gz_tags = gzopen(tag_file.c_str(), "wb");
         if (!gz_tags) {
@@ -1090,13 +1079,24 @@ int write_catalog(map<int, CLocus *> &catalog) {
         #if ZLIB_VERNUM >= 0x1240
         gzbuffer(gz_alle, libz_buffer_size);
         #endif
+
+	gz_sams = gzopen(sam_file.c_str(), "wb");
+        if (!gz_sams) {
+            cerr << "Error: Unable to open gzipped catalog sample list file '" << sam_file << "': " << strerror(errno) << ".\n";
+            exit(1);
+        }
+        #if ZLIB_VERNUM >= 0x1240
+        gzbuffer(gz_sams, libz_buffer_size);
+        #endif
     } else {
         tags.open(tag_file.c_str());
         snps.open(snp_file.c_str());
         alle.open(all_file.c_str());
+	sams.open(sam_file.c_str());
         check_open(tags, tag_file);
         check_open(snps, snp_file);
         check_open(alle, all_file);
+	check_open(sams, sam_file);
     }
 
     //
@@ -1116,10 +1116,12 @@ int write_catalog(map<int, CLocus *> &catalog) {
         gzputs_throwing(gz_tags, log.str().c_str());
         gzputs_throwing(gz_snps, log.str().c_str());
         gzputs_throwing(gz_alle, log.str().c_str());
+	gzputs_throwing(gz_sams, log.str().c_str());
     } else {
         tags << log.str();
         snps << log.str();
         alle << log.str();
+	sams << log.str();
     }
 
     for (i = catalog.begin(); i != catalog.end(); i++) {
@@ -1131,6 +1133,31 @@ int write_catalog(map<int, CLocus *> &catalog) {
             write_simple_output(tag, tags, snps, alle);
     }
 
+    //
+    // Write the list of samples incorporated into the catalog, sorted by sample ID.
+    //
+    vector<pair<string, size_t>> sorted_seen_samples;
+    for (auto it = seen_samples.begin(); it != seen_samples.end(); it++) {
+	string sample_path = it->first;
+	// Remove the file path off the sample string.
+	int pos = sample_path.find_last_of("/");
+	sample_path = sample_path.substr(pos+1);
+	sorted_seen_samples.push_back(make_pair(sample_path, it->second));
+    }
+    sort(sorted_seen_samples.begin(), sorted_seen_samples.end(),
+	 [] (pair<string, size_t> i, pair<string, size_t> j) { return (i.second < j.second); });
+
+    for (auto it = sorted_seen_samples.begin(); it != sorted_seen_samples.end(); it++) {
+	log.str("");
+	log << it->first << "\t" << it->second << "\n";
+
+	if (gzip) {
+	    gzputs_throwing(gz_sams, log.str().c_str());
+	} else {
+	    sams << log.str();
+	}
+    }	
+    
     time(&rawtime);
     timeinfo = localtime(&rawtime);
     strftime(date, 32, "%F %T", timeinfo);
@@ -1150,10 +1177,12 @@ int write_catalog(map<int, CLocus *> &catalog) {
         gzclose_throwing(gz_tags);
         gzclose_throwing(gz_snps);
         gzclose_throwing(gz_alle);
+	gzclose_throwing(gz_sams);
     } else {
         tags.close();
         snps.close();
         alle.close();
+	sams.close();
     }
 
     return 0;
@@ -1613,8 +1642,7 @@ write_simple_output(CLocus *tag, ofstream &cat_file, ofstream &snp_file, ofstrea
     }
     sources = sources.substr(0, sources.length() - 1);
 
-    cat_file << 0           << "\t" // Catalog has no sample ID.
-             << tag->id     << "\t"
+    cat_file << tag->id     << "\t"
              << "consensus" << "\t"
              << "0"         << "\t" // Catalog has no component number for stacks since only consensus sequences are stored.
              << sources     << "\t"
@@ -1627,8 +1655,7 @@ write_simple_output(CLocus *tag, ofstream &cat_file, ofstream &snp_file, ofstrea
     // Output the SNPs associated with the catalog tag
     //
     for (auto snp_it = tag->snps.begin(); snp_it != tag->snps.end(); snp_it++) {
-        snp_file << "0"            << "\t"
-                 << tag->id        << "\t"
+	snp_file << tag->id        << "\t"
                  << (*snp_it)->col << "\t";
 
         switch((*snp_it)->type) {
@@ -1655,8 +1682,7 @@ write_simple_output(CLocus *tag, ofstream &cat_file, ofstream &snp_file, ofstrea
     // Output the alleles associated with the two matched tags
     //
     for (auto all_it = tag->alleles.begin(); all_it != tag->alleles.end(); all_it++)
-        all_file << "0"           << "\t"
-                 << tag->id       << "\t"
+	all_file << tag->id       << "\t"
                  << all_it->first << "\t"
                  << "0"           << "\t"    // These two fields are used in the
                  << "0"           << "\n";   // ustacks/pstacks output, not in cstacks.
@@ -1678,8 +1704,7 @@ write_gzip_output(CLocus *tag, gzFile &cat_file, gzFile &snp_file, gzFile &all_f
 
     sstr.str("");
 
-    sstr << 0           << "\t" // Catalog has no sample ID.
-         << tag->id     << "\t"
+    sstr << tag->id     << "\t"
          << "consensus" << "\t"
          << "0"         << "\t" // Catalog has no component number for stacks since only consensus sequences are stored.
          << sources     << "\t"
@@ -1695,8 +1720,7 @@ write_gzip_output(CLocus *tag, gzFile &cat_file, gzFile &snp_file, gzFile &all_f
     // Output the SNPs associated with the catalog tag
     //
     for (auto snp_it = tag->snps.begin(); snp_it != tag->snps.end(); snp_it++) {
-        sstr << "0"            << "\t"
-             <<   tag->id      << "\t"
+	sstr <<   tag->id      << "\t"
              << (*snp_it)->col << "\t";
 
         switch((*snp_it)->type) {
@@ -1726,8 +1750,7 @@ write_gzip_output(CLocus *tag, gzFile &cat_file, gzFile &snp_file, gzFile &all_f
     // Output the alleles associated with the two matched tags
     //
     for (auto all_it = tag->alleles.begin(); all_it != tag->alleles.end(); all_it++)
-        sstr << "0"     << "\t"
-             << tag->id << "\t"
+	sstr << tag->id << "\t"
              << all_it->first << "\t"
              << 0       << "\t"
              << 0       << "\n";
@@ -1738,7 +1761,8 @@ write_gzip_output(CLocus *tag, gzFile &cat_file, gzFile &snp_file, gzFile &all_f
 }
 
 int
-initialize_new_catalog(pair<int, string> &sample, map<int, CLocus *> &catalog)
+initialize_new_catalog(pair<int, string> &sample, map<int, CLocus *> &catalog,
+		       map<string, size_t> &seen_samples, size_t &next_sample_id)
 {
     map<int, CLocus *> tmp_catalog;
     bool compressed = false;
@@ -1751,7 +1775,7 @@ initialize_new_catalog(pair<int, string> &sample, map<int, CLocus *> &catalog)
 
     in_file_type = compressed == true ? FileT::gzsql : FileT::sql;
 
-    sample.first = tmp_catalog.begin()->second->sample_id;
+    sample.first = next_sample_id;
 
     //
     // Iterate over the catalog entires and renumber them after recording the source of
@@ -1768,11 +1792,15 @@ initialize_new_catalog(pair<int, string> &sample, map<int, CLocus *> &catalog)
 
     cerr << "  " << catalog.size() << " loci were newly added to the catalog.\n";
 
+    seen_samples[sample.second] = next_sample_id;
+    next_sample_id++;
+
     return 1;
 }
 
 int
-initialize_existing_catalog(string catalog_path, map<int, CLocus *> &catalog)
+initialize_existing_catalog(string catalog_path, map<int, CLocus *> &catalog,
+			    map<string, size_t> &seen_samples, size_t &next_sample_id)
 {
     bool compressed;
 
@@ -1823,6 +1851,13 @@ initialize_existing_catalog(string catalog_path, map<int, CLocus *> &catalog)
 
     next_catalog_id = max_catalog_id + 1;
 
+    //
+    // Parse the list of samples in the existing catalog.
+    //
+    if (!load_catalog_samples(catalog_path, seen_samples, next_sample_id))
+	throw exception();
+    next_sample_id++;
+
     return 1;
 }
 
