@@ -934,12 +934,14 @@ build_file_list(vector<pair<string, string> > &files)
             // Check the file suffix to make sure we should process it.
             //
             p   = file.c_str();
-            q   = p + file.length() + 1;
+            q   = p + file.length();
             end = q;
             while (q >= p && *q != '.') q--;
-            if (strcmp(q, ".gz") == 0) {
+            if (strcmp(q, ".gz") == 0 && q > p) {
                 end = q;
-                while (q >= p && *q != '.') q--;
+                do {
+                    q--;
+                } while (q >= p && *q != '.');
             }
             if (strncmp(q, ".fq",    end - q) != 0 &&
                 strncmp(q, ".fa",    end - q) != 0 &&
@@ -969,6 +971,8 @@ build_file_list(vector<pair<string, string> > &files)
         if (files.size() == 0) {
             cerr << "Unable to locate any input files to process within '" << in_path_1 << "'\n";
         }
+        closedir(dir);
+        
     } else {
         //
         // Files specified directly:
@@ -1002,3 +1006,31 @@ build_file_list(vector<pair<string, string> > &files)
 
     return 0;
 }
+
+int
+file_suffix_position(string filename)
+{
+    const char *p, *q, *end;
+    
+    //
+    // Check the file suffix to make sure we should process it.
+    //
+    p   = filename.c_str();
+    q   = p + filename.length();
+    end = q;
+    while (q >= p && *q != '.') q--;
+    if (strcmp(q, ".gz") == 0 && q > p) {
+        end = q;
+        do {
+            q--;
+        } while (q >= p && *q != '.');
+    }
+    if (strncmp(q, ".fq",    end - q) == 0 ||
+        strncmp(q, ".fa",    end - q) == 0 ||
+        strncmp(q, ".fastq", end - q) == 0 ||
+        strncmp(q, ".fasta", end - q) == 0 ||
+        strncmp(q, ".bam",   end - q) == 0)
+        return q - p;
+    else
+        return end - p;
+}
