@@ -52,6 +52,7 @@ barcodet barcode_type    = null_null;
 bool     retain_header   = false;
 bool     filter_adapter  = false;
 bool     bestrad         = false;
+bool     methylrad       = false;
 bool     paired          = false;
 bool     clean           = false;
 bool     quality         = false;
@@ -109,6 +110,28 @@ int main (int argc, char* argv[]) {
 
     parse_command_line(argc, argv);
 
+    //
+    // If the user speifies a methylated restriction enzyme, make sure we have its sequence.
+    //
+    if (check_radtag && methylrad) {
+	initialize_renz_methylconv(renz);
+
+	if (renz.count(renz_1) == 0) {
+	    cerr << "Unrecognized methylated restriction enzyme specified: '" << renz_1.c_str() << "'.\n";
+	    help();
+	}
+
+	if (renz_2.length() > 0 && renz.count(renz_2) == 0) {
+	    cerr << "Unrecognized methylated restriction enzyme specified: '" << renz_2.c_str() << "'.\n";
+	    help();
+	}
+
+	cerr << "Converting restriction enzyme sequence for '" << renz_1 << "'";
+	if (renz_2.length() > 0)
+	    cerr << " and '" << renz_2 << "'";
+	cerr << " to methylated version for cut site remnant checking.\n";
+    }
+    
     //
     // If input files are gzipped, output gziped files, unless the user chooses an output type.
     //
@@ -415,7 +438,11 @@ process_paired_reads(string prefix_1,
     // Open a file for recording discarded reads
     //
     if (discards) {
-        path_1 = out_path + prefix_1 + ".discards";
+        string dprefix_1 = prefix_1.substr(0, file_suffix_position(prefix_1));
+        string dprefix_2 = prefix_2.substr(0, file_suffix_position(prefix_2));
+        string dsuffix   = (out_file_type == FileT::fastq || out_file_type == FileT::gzfastq) ? ".fq" : ".fa";
+
+        path_1 = out_path + dprefix_1 + ".discards" + dsuffix;
         discard_fh_1 = new ofstream(path_1.c_str(), ifstream::out);
 
         if (discard_fh_1->fail()) {
@@ -423,7 +450,7 @@ process_paired_reads(string prefix_1,
             exit(1);
         }
 
-        path_2 = out_path + prefix_2 + ".discards";
+        path_2 = out_path + dprefix_2 + ".discards" + dsuffix;
         discard_fh_2 = new ofstream(path_2.c_str(), ifstream::out);
 
         if (discard_fh_2->fail()) {
@@ -638,7 +665,7 @@ process_paired_reads_parallel(string prefix_1,
 			      map<BarcodePair, map<string, long> > &barcode_log) {
     Input    *fh_1=NULL, *fh_2=NULL;
     RawRead   *r_1=NULL,  *r_2=NULL;
-    ofstream *discard_fh_1=NULL, *discard_fh_2=NULL;
+    vector<ofstream *> discard_fhs_1(num_output_threads), discard_fhs_2(num_output_threads);
 
     int return_val = 1;
 
@@ -668,20 +695,29 @@ process_paired_reads_parallel(string prefix_1,
     // Open a file for recording discarded reads
     //
     if (discards) {
-        path_1 = out_path + prefix_1 + ".discards";
-        discard_fh_1 = new ofstream(path_1.c_str(), ifstream::out);
+        string dprefix_1 = prefix_1.substr(0, file_suffix_position(prefix_1));
+        string dprefix_2 = prefix_2.substr(0, file_suffix_position(prefix_2));
 
-        if (discard_fh_1->fail()) {
-            cerr << "Error opening single-end discard output file '" << path_1 << "'\n";
-            exit(1);
-        }
+        string dsuffix = (out_file_type == FileT::fastq || out_file_type == FileT::gzfastq) ? ".fq" : ".fa";
+        
+        for (uint i = 0; i < discard_fhs_1.size(); i++) {
+            string thread_str = discard_fhs_1.size() > 1 ? "." + to_string(i+1) : "";
+            
+            path_1 = out_path + dprefix_1 + thread_str + ".discards" + dsuffix;
+            discard_fhs_1[i] = new ofstream(path_1.c_str(), ifstream::out);
 
-        path_2 = out_path + prefix_2 + ".discards";
-        discard_fh_2 = new ofstream(path_2.c_str(), ifstream::out);
+            if (discard_fhs_1[i]->fail()) {
+                cerr << "Error opening single-end discard output file '" << path_1 << "'\n";
+                exit(1);
+            }
 
-        if (discard_fh_2->fail()) {
-            cerr << "Error opening paired-end discard output file '" << path_2 << "'\n";
-            exit(1);
+            path_2 = out_path + dprefix_2 + thread_str + ".discards" + dsuffix;
+            discard_fhs_2[i] = new ofstream(path_2.c_str(), ifstream::out);
+
+            if (discard_fhs_2[i]->fail()) {
+                cerr << "Error opening paired-end discard output file '" << path_2 << "'\n";
+                exit(1);
+            }
         }
     }
 
@@ -890,12 +926,12 @@ process_paired_reads_parallel(string prefix_1,
 
 		if (discards && !r_1->retain)
 		    result_1 = (out_file_type == FileT::fastq || out_file_type == FileT::gzfastq) ?
-			write_fastq(discard_fh_1, s_1) :
-			write_fasta(discard_fh_1, s_1);
+			write_fastq(discard_fhs_1[thread_no - 1], s_1) :
+			write_fasta(discard_fhs_1[thread_no - 1], s_1);
 		if (discards && !r_2->retain)
 		    result_2 = (out_file_type == FileT::fastq || out_file_type == FileT::gzfastq) ?
-			write_fastq(discard_fh_2, s_2) :
-			write_fasta(discard_fh_2, s_2);
+			write_fastq(discard_fhs_2[thread_no - 1], s_2) :
+			write_fasta(discard_fhs_2[thread_no - 1], s_2);
 
 		if (!result_1 || !result_2) {
 		    cerr << "Error writing to discard file for '" << bc->str() << "'\n";
@@ -1060,8 +1096,10 @@ process_paired_reads_parallel(string prefix_1,
     consolidate_counters(counter, local_counters);
 
     if (discards) {
-        delete discard_fh_1;
-        delete discard_fh_2;
+        for (uint i = 0; i < discard_fhs_1.size(); i++) {
+            delete discard_fhs_1[i];
+            delete discard_fhs_2[i];
+        }
     }
 
     delete fh_1;
@@ -1101,7 +1139,10 @@ process_reads(string prefix,
     // Open a file for recording discarded reads
     //
     if (discards) {
-        path = out_path + prefix + ".discards";
+        string dprefix = prefix.substr(0, file_suffix_position(prefix));
+        string dsuffix = (out_file_type == FileT::fastq || out_file_type == FileT::gzfastq) ? ".fq" : ".fa";
+
+        path = out_path + dprefix + ".discards" + dsuffix;
         discard_fh = new ofstream(path.c_str(), ifstream::out);
 
         if (discard_fh->fail()) {
@@ -1242,8 +1283,8 @@ process_reads_parallel(string prefix,
 		       map<string, long> &counter,
 		       map<BarcodePair, map<string, long>> &barcode_log)
 {
-    Input    *fh         = NULL;
-    ofstream *discard_fh = NULL;
+    Input *fh = NULL;
+    vector<ofstream *> discard_fhs(num_output_threads);
 
     int return_val = 1;
 
@@ -1262,12 +1303,18 @@ process_reads_parallel(string prefix,
     // Open a file for recording discarded reads
     //
     if (discards) {
-        path = out_path + prefix + ".discards";
-        discard_fh = new ofstream(path.c_str(), ifstream::out);
+        string dprefix = prefix.substr(0, file_suffix_position(prefix));
+        string dsuffix = (out_file_type == FileT::fastq || out_file_type == FileT::gzfastq) ? ".fq" : ".fa";
 
-        if (discard_fh->fail()) {
-            cerr << "Error opening discard output file '" << path << "'\n";
-            exit(1);
+        for (uint i = 0; i < discard_fhs.size(); i++) {
+            string thread_str = discard_fhs.size() > 1 ? + "." + to_string(i+1) : "";
+            path = out_path + dprefix + thread_str +".discards" + dsuffix;
+            discard_fhs[i] = new ofstream(path.c_str(), ifstream::out);
+
+            if (discard_fhs[i]->fail()) {
+                cerr << "Error opening discard output file '" << path << "'\n";
+                exit(1);
+            }
         }
     }
 
@@ -1439,10 +1486,9 @@ process_reads_parallel(string prefix,
 		}
 
 		if (discards && !r->retain) {
-		    #pragma omp critical
 		    result = out_file_type == FileT::fastq || out_file_type == FileT::gzfastq ?
-			write_fastq(discard_fh, s) :
-			write_fasta(discard_fh, s);
+			write_fastq(discard_fhs[thread_no - 1], s) :
+			write_fasta(discard_fhs[thread_no - 1], s);
 		}
 		
 		if (!result) {
@@ -1572,8 +1618,12 @@ process_reads_parallel(string prefix,
     consolidate_barcode_logs(barcode_log, local_barcode_log);
     consolidate_counters(counter, local_counters);
     
-    if (discards) delete discard_fh;
-
+    if (discards) {
+        for (uint i = 0; i < discard_fhs.size(); i++) {
+            delete discard_fhs[i];
+        }
+    }
+    
     //
     // Close the file and delete the Input object.
     //
@@ -2092,9 +2142,10 @@ int parse_command_line(int argc, char* argv[]) {
             {"interleaved",          no_argument, NULL, 'I'},
             {"merge",                no_argument, NULL, 'm'},
             {"disable-rad-check",    no_argument, NULL, 'R'}, {"disable_rad_check",    no_argument, NULL, 'R'},
-            {"filter-illumina",      no_argument, NULL, 'F'}, {"filter_illumina",      no_argument, NULL, 'F'},
-            {"retain-header",        no_argument, NULL, 'H'}, {"retain_header",        no_argument, NULL, 'H'},
+            {"filter-illumina",      no_argument, NULL, 'F'},
+            {"retain-header",        no_argument, NULL, 'H'},
             {"bestrad",              no_argument, NULL, 1000},
+	    {"methylrad",            no_argument, NULL, 1002},
             {"null-index",           no_argument, NULL, 'U'}, {"null_index",           no_argument, NULL, 'U'},
             {"index-null",           no_argument, NULL, 'u'}, {"index_null",           no_argument, NULL, 'u'},
             {"inline-null",          no_argument, NULL, 'V'}, {"inline_null",          no_argument, NULL, 'V'},
@@ -2196,6 +2247,9 @@ int parse_command_line(int argc, char* argv[]) {
 	case 1001:
 	    num_threads = is_integer(optarg);
 	    break;
+	case 1002:
+            methylrad = true;
+            break;
         case 'B':
             barcode_dist_1 = is_integer(optarg);
             break;
@@ -2471,7 +2525,8 @@ void help() {
 
     cerr << "\n"
          << "  Protocol-specific options:\n"
-         << "    --bestrad: library was generated using BestRAD, check for restriction enzyme on either read and potentially tranpose reads.\n\n"
+         << "    --bestrad: library was generated using BestRAD, check for restriction enzyme on either read and potentially tranpose reads.\n"
+	 << "    --methylrad: library was generated using enzymatic methyl-seq (EM-seq) or bisulphite sequencing.\n\n"
          << "  Adapter options:\n"
          << "    --adapter-1 <sequence>: provide adaptor sequence that may occur on the single-end read for filtering.\n"
          << "    --adapter-2 <sequence>: provide adaptor sequence that may occur on the paired-read for filtering.\n"
