@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2010-2020, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2010-2022, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -71,7 +71,6 @@ try{
 
     cerr << "ustacks parameters selected:\n"
          << "  Input file: '" << in_file   << "'\n"
-         << "  Sample ID: "   << sample_id << "\n"
          << "  Min depth of coverage to create a stack (m): " << min_merge_cov << "\n"
          << "  Repeat removal algorithm: " << (remove_rep_stacks ? "enabled" : "disabled") << "\n"
          << "  Max distance allowed between stacks (M): " << max_utag_dist << "\n"
@@ -115,6 +114,10 @@ try{
     // Set the number of OpenMP parallel threads to execute.
     //
     #ifdef _OPENMP
+    if (num_threads > max_threads) {
+	num_threads = max_threads;
+	cerr << "Warning: too many threads requested and performance will be negatively affected; reducing thread count to " << num_threads << ".\n";
+    }
     omp_set_num_threads(num_threads);
     #endif
 
@@ -181,7 +184,7 @@ try{
     //
     // Assemble loci (merge primary stacks).
     //
-    cerr << "Assembling stacks (max. dist. M=" << max_utag_dist << ")...\n";
+    cerr << "Assembling stacks (max dist. M=" << max_utag_dist << ")...\n";
     calc_kmer_distance(merged, max_utag_dist);
     size_t n_blacklisted;
     merge_stacks(merged, n_blacklisted);
@@ -196,8 +199,9 @@ try{
     //
     // Merge secondary stacks.
     //
-    cerr << "Merging secondary stacks (max. dist. N=" << max_rem_dist << " from consensus)...\n";
+    cerr << "Merging secondary stacks (max dist. N=" << max_rem_dist << " from consensus)...\n";
     size_t utilized    = merge_remainders(merged, unique, remainders);
+    call_consensus(merged, unique, remainders, false);
     size_t util_gapped = merge_gapped_remainders(merged, unique, remainders);
     call_consensus(merged, unique, remainders, false);
     cerr << "  Merged " << utilized+util_gapped << " out of " << n_r_reads << " secondary reads ("
@@ -239,9 +243,10 @@ try{
     }
 
     //
-    // Report how many reads were used.
+    // Report how many loci were built, how many reads were used, and final coverage.
     //
-    cerr << "Final coverage: ";
+    cerr << "Final number of stacks built: " << merged.size() << "\n"
+	 << "Final coverage: ";
     report_cov();
     cerr << "\n";
 
@@ -254,10 +259,20 @@ try{
     //
     // Write output files.
     //
-    cerr << "Writing tags, SNPs, and alleles files...\n";
+    cerr << "Writing tags, SNPs, and alleles files to '" << prefix_path << "'...";
     write_results(merged, unique, remainders);
     cerr << "done.\n";
 
+    //
+    // Free remaining memory.
+    //
+    for (auto it = unique.begin(); it != unique.end(); it++)
+        delete it->second;
+    for (auto it = remainders.begin(); it != remainders.end(); it++)
+        delete it->second;
+    for (auto it = merged.begin(); it != merged.end(); it++)
+        delete it->second;
+    
     cerr << "ustacks is done.\n";
     return 0;
 } catch(std::exception& e) {
@@ -305,14 +320,13 @@ merge_gapped_alns(map<int, Stack *> &unique, map<int, Rem *> &rem, map<int, Merg
         if (tag_2->alns.size() > 1 && tag_2->alns[0].pct_id == tag_2->alns[1].pct_id)
             continue;
 
-        if (tag_1->id != (int)tag_2->alns[0].id)
+        if (tag_1->id != (int) tag_2->alns[0].id)
             continue;
 
         cigar_1 = invert_cigar(tag_1->alns[0].cigar);
         cigar_2 = tag_2->alns[0].cigar;
 
         if (cigar_1 == cigar_2) {
-            cerr << "CIGAR: " << cigar_1 << "\n";
             parse_cigar(tag_1->alns[0].cigar.c_str(), cigar);
 
             //
@@ -400,9 +414,10 @@ edit_gapped_seqs(map<int, Stack *> &unique, map<int, Rem *> &rem, MergedStack *t
     int    stack_id;
     Stack *s;
     Rem   *r;
-    char  *buf = new char[cigar_length_padded(cigar) + 1];
     string seq;
 
+    char *buf = new char[cigar_length_padded(cigar) + 1];
+    
     for (uint i = 0; i < tag->utags.size(); i++) {
         stack_id = tag->utags[i];
         s = unique[stack_id];
@@ -417,7 +432,7 @@ edit_gapped_seqs(map<int, Stack *> &unique, map<int, Rem *> &rem, MergedStack *t
     for (uint i = 0; i < tag->remtags.size(); i++) {
         stack_id = tag->remtags[i];
         r = rem[stack_id];
-
+        
         r->seq->seq(buf);
         seq = apply_cigar_to_seq(buf, cigar);
 
@@ -1025,25 +1040,7 @@ merge_gapped_remainders(map<int, MergedStack *> &merged, map<int, Stack *> &uniq
             for (uint i = 0; i < tag_1->rem_queue.size(); i++) {
                 Rem *r = rem.at(tag_1->rem_queue[i]);
 
-                // //
-                // // We want to align the last max_rem_dist * 2 nucleotides. If sequences are already the wrong length,
-                // // adjust so the 3' ends are lined up properly in the alignment matrix.
-                // //
-                // diff    = tag_1->len - r->seq->size();
-                // q_start = r->seq->size() - (max_rem_dist * 2) - 1;
-                // q_end   = r->seq->size() - 1;
-                // s_start = tag_1->len - (max_rem_dist * 2) - 1;
-                // s_end   = tag_1->len - 1;
-
-                // q_start = diff > 0 ? q_start + diff : q_start;
-                // s_start = diff < 0 ? s_start + abs(diff) : s_start;
-
-                // cerr << "Consensus size: " << tag_1->len << "; remainder size: " << r->seq->size() << "\n"
-                //      << "Con seq: " << tag_1->con << " (" << strlen(tag_1->con) << ")\n"
-                //      << "Rem seq: " << r->seq->seq() << "\n"
-                //      << "q_start: " << q_start << "; q_end: " << q_end << "; s_start: " << s_start << "; s_end: " << s_end << "\n";
-
-                aln->init(r->seq->size(), tag_1->len);
+                aln->init(r->seq->size(), tag_1->len, true);
 
                 if (aln->align(r->seq->seq().c_str(), tag_1->con)) {
                     a = aln->result();
@@ -1092,20 +1089,27 @@ merge_gapped_remainders(map<int, MergedStack *> &merged, map<int, Stack *> &uniq
                         }
                         pos += cigar[j].second;
                     }
-                    edit_gapped_seqs(unique, rem, tag_1, cigar);
-
                     //
-                    // Add this aligned remainder tag to the locus after adjusting the other reads but before adjusting the consensus.
+                    // 1. Push newly created alignment gaps into existing constituent primary and remainder sequences.
+                    //
+                    edit_gapped_seqs(unique, rem, tag_1, cigar);
+                    //
+                    // 2. Add this aligned remainder tag to the locus after adjusting the other reads but before adjusting the consensus.
                     //
                     tag_1->remtags.push_back(r->id);
+                    //
+                    // 3. Adjust the consensus.
+                    //
                     update_consensus(tag_1, unique, rem);
                 }
             }
 
             tag_1->rem_queue.clear();
         }
-    }
 
+        delete aln;
+    }
+        
     return utilized;
 }
 
@@ -1117,24 +1121,27 @@ call_alleles(MergedStack *mtag, vector<DNANSeq *> &reads, vector<read_type> &rea
     string  allele;
     DNANSeq *d;
     char    base;
+    size_t  snp_cnt;
     vector<SNP *>::iterator snp;
 
     for (row = 0; row < height; row++) {
-        allele.clear();
-
-        uint snp_cnt = 0;
-
         //
         // Only call a haplotype from primary reads.
         //
         if (!call_sec_hapl && read_types[row] == secondary) continue;
 
+        allele.clear();
+        snp_cnt = 0;
+        d = reads[row];
+
         for (snp = mtag->snps.begin(); snp != mtag->snps.end(); snp++) {
             if ((*snp)->type != snp_type_het) continue;
 
             snp_cnt++;
 
-            d    = reads[row];
+            if ((*snp)->col >= d->size())
+                break;
+
             base = (*d)[(*snp)->col];
 
             //
@@ -1183,30 +1190,34 @@ call_consensus(map<int, MergedStack *> &merged, map<int, Stack *> &unique, map<i
             // that tag into our array as many times as it originally occurred.
             //
             vector<int>::iterator j;
-            vector<DNANSeq *> reads;
-            vector<read_type> read_types;
-            uint length = unique[mtag->utags.front()]->seq->size();
-
+            vector<DNANSeq *>     reads;
+            vector<read_type>     read_types;
+            size_t                length = 0;
+            
             for (j = mtag->utags.begin(); j != mtag->utags.end(); j++) {
                 utag = unique[*j];
 
+                // Check for maximum length.
+                length = utag->seq->size() > length ? utag->seq->size() : length;
+
                 for (uint k = 0; k < utag->count(); k++) {
                     reads.push_back(utag->seq);
                     read_types.push_back(primary);
-
-                    assert(utag->seq->size() == length);
                 }
             }
 
+            //
             // For each remainder tag that has been merged into this Stack, add the sequence.
+            //
             for (j = mtag->remtags.begin(); j != mtag->remtags.end(); j++) {
                 r = rem[*j];
 
+                // Check for maximum length.
+                length = r->seq->size() > length ? r->seq->size() : length;
+
                 for (uint k = 0; k < r->count(); k++) {
                     reads.push_back(r->seq);
                     read_types.push_back(secondary);
-
-                    assert(r->seq->size() == length);
                 }
             }
 
@@ -1230,7 +1241,7 @@ call_consensus(map<int, MergedStack *> &merged, map<int, Stack *> &unique, map<i
 
                 for (row = 0; row < height; row++) {
                     d = reads[row];
-                    if (nuc.count((*d)[col]))
+                    if (col < d->size() && nuc.count((*d)[col]))
                         nuc[(*d)[col]]++;
                 }
 
@@ -1267,13 +1278,13 @@ call_consensus(map<int, MergedStack *> &merged, map<int, Stack *> &unique, map<i
                     }
 
                     switch(model_type) {
-                    case snp:
+                    case modelt::snp:
                         call_multinomial_snp(mtag, col, nuc, true);
                         break;
-                    case bounded:
+                    case modelt::bounded:
                         call_bounded_multinomial_snp(mtag, col, nuc, true);
                         break;
-                    case ::fixed:
+                    case modelt::fixed:
                         call_multinomial_fixed(mtag, col, nuc);
                         break;
                     default:
@@ -1290,7 +1301,7 @@ call_consensus(map<int, MergedStack *> &merged, map<int, Stack *> &unique, map<i
 
                 call_alleles(mtag, reads, read_types);
 
-                if (model_type == ::fixed) {
+                if (model_type == modelt::fixed) {
                     //
                     // Mask nucleotides that are not fixed.
                     //
@@ -1314,6 +1325,8 @@ update_consensus(MergedStack *mtag, map<int, Stack *> &unique, map<int, Rem *> &
 {
     Stack *utag;
     Rem   *r;
+    size_t length = 0;
+    
     //
     // Create a two-dimensional array, each row containing one read. For
     // each unique tag that has been merged together, add the sequence for
@@ -1324,6 +1337,9 @@ update_consensus(MergedStack *mtag, map<int, Stack *> &unique, map<int, Rem *> &
     for (auto j = mtag->utags.begin(); j != mtag->utags.end(); j++) {
         utag = unique[*j];
 
+        // Check for maximum length.
+        length = utag->seq->size() > length ? utag->seq->size() : length;
+
         for (uint k = 0; k < utag->count(); k++)
             reads.push_back(utag->seq);
     }
@@ -1332,6 +1348,9 @@ update_consensus(MergedStack *mtag, map<int, Stack *> &unique, map<int, Rem *> &
     for (auto j = mtag->remtags.begin(); j != mtag->remtags.end(); j++) {
         r = rem[*j];
 
+        // Check for maximum length.
+        length = r->seq->size() > length ? r->seq->size() : length;
+
         for (uint k = 0; k < r->count(); k++)
             reads.push_back(r->seq);
     }
@@ -1341,7 +1360,6 @@ update_consensus(MergedStack *mtag, map<int, Stack *> &unique, map<int, Rem *> &
     //
     string   con;
     uint     row, col;
-    uint     length = reads[0]->size();
     uint     height = reads.size();
     DNANSeq *d;
 
@@ -1358,6 +1376,10 @@ update_consensus(MergedStack *mtag, map<int, Stack *> &unique, map<int, Rem *> &
 
         for (row = 0; row < height; row++) {
             d = reads[row];
+
+            if (col >= d->size())
+                continue;
+            
             switch ((*d)[col]) {
             case 'A':
                 nucs[0]++;
@@ -1672,7 +1694,9 @@ merge_tags(MergedStack *tag_1, MergedStack *tag_2, int id)
     return new_tag;
 }
 
-MergedStack *merge_tags(map<int, MergedStack *> &merged, set<int> &merge_list, int id) {
+MergedStack *
+merge_tags(map<int, MergedStack *> &merged, set<int> &merge_list, int id)
+{
     MergedStack *tag_1, *tag_2;
 
     tag_1     = new MergedStack;
@@ -1701,7 +1725,9 @@ MergedStack *merge_tags(map<int, MergedStack *> &merged, set<int> &merge_list, i
     return tag_1;
 }
 
-MergedStack *merge_tags(map<int, MergedStack *> &merged, int *merge_list, int merge_list_size, int id) {
+MergedStack *
+merge_tags(map<int, MergedStack *> &merged, int *merge_list, int merge_list_size, int id)
+{
     int                   i;
     vector<int>::iterator j;
     MergedStack *tag_1, *tag_2;
@@ -2351,8 +2377,7 @@ write_results(map<int, MergedStack *> &m, map<int, Stack *> &u, map<int, Rem *>
         tag_1->calc_likelihood();
 
         // First write the consensus sequence
-        tags << sample_id          << "\t"
-             << tag_1->id          << "\t"
+	tags  << tag_1->id          << "\t"
              << "consensus\t"      << "\t"
              << "\t"
              << tag_1->con         << "\t"
@@ -2363,8 +2388,7 @@ write_results(map<int, MergedStack *> &m, map<int, Stack *> &u, map<int, Rem *>
         //
         // Write a sequence recording the output of the SNP model for each nucleotide.
         //
-        tags << sample_id << "\t"
-             << tag_1->id << "\t"
+	tags << tag_1->id << "\t"
              << "model\t" << "\t"
              << "\t";
         for (s = tag_1->snps.begin(); s != tag_1->snps.end(); s++) {
@@ -2391,8 +2415,7 @@ write_results(map<int, MergedStack *> &m, map<int, Stack *> &u, map<int, Rem *>
             total += tag_2->count();
 
             for (uint j = 0; j < tag_2->map.size(); j++) {
-                tags << sample_id << "\t"
-                     << tag_1->id << "\t"
+		tags << tag_1->id << "\t"
                      << "primary\t"
                      << id << "\t"
                      << seq_ids[tag_2->map[j]] << "\t"
@@ -2409,8 +2432,7 @@ write_results(map<int, MergedStack *> &m, map<int, Stack *> &u, map<int, Rem *>
             rem    = r[*k];
             total += rem->map.size();
             for (uint j = 0; j < rem->map.size(); j++)
-                tags << sample_id << "\t"
-                     << tag_1->id << "\t"
+		tags << tag_1->id << "\t"
                      << "secondary\t"
                      << "\t"
                      << seq_ids[rem->map[j]] << "\t"
@@ -2422,8 +2444,7 @@ write_results(map<int, MergedStack *> &m, map<int, Stack *> &u, map<int, Rem *>
         // Write out the model calls for each nucleotide in this locus.
         //
         for (s = tag_1->snps.begin(); s != tag_1->snps.end(); s++) {
-            snps << sample_id << "\t"
-                 << tag_1->id << "\t"
+	    snps << tag_1->id << "\t"
                  << (size_t) (*s)->col << "\t";
             switch((*s)->type) {
             case snp_type_het:
@@ -2447,8 +2468,7 @@ write_results(map<int, MergedStack *> &m, map<int, Stack *> &u, map<int, Rem *>
         // the percentage of tags a particular allele occupies.
         //
         for (t = tag_1->alleles.begin(); t != tag_1->alleles.end(); t++) {
-            alle << sample_id   << "\t"
-                 << tag_1->id   << "\t"
+	    alle << tag_1->id   << "\t"
                  << (*t).first  << "\t";
             sprintf(strbuf32, "%.2f", ((*t).second/total) * 100);
             alle << strbuf32 << "\t"
@@ -2703,7 +2723,7 @@ void load_radtags(string in_file, DNASeqHashMap &radtags, size_t& n_reads) {
         cerr << "Error: different sequence lengths detected, this will interfere with Stacks "
              << "algorithms, trim reads to uniform length (override this check with --force-diff-len).\n";
         exit(1);
-    } else if (force_diff_len) {
+    } else if (len_mismatch && force_diff_len) {
         cerr << "Warning: different sequence lengths detected, this could interfere with Stacks algorithms.\n";
     }
     if (corrected > 0)
@@ -2727,8 +2747,6 @@ load_seq_ids(vector<char *> &seq_ids)
     else if (in_file_type == FileT::gzfastq)
         fh = new GzFastq(in_file.c_str());
 
-    cerr << "Refetching read IDs...";
-
     char *id;
     Seq c;
     c.id   = new char[id_len];
@@ -2834,7 +2852,6 @@ int parse_command_line(int argc, char* argv[]) {
             sample_name = optarg;
             break;
         case 'i':
-            sample_id = is_integer(optarg);
             break;
         case 'm':
             min_merge_cov = is_integer(optarg);
@@ -2991,11 +3008,6 @@ int parse_command_line(int argc, char* argv[]) {
     }
     prefix_path = out_path + sample_name;
 
-    if (sample_id < 0) {
-        cerr << "A unique sample ID must be provided (a positive integer).\n";
-        help();
-    }
-
     if (model_type == ::fixed && barcode_err_freq == 0) {
         cerr << "You must specify the barcode error frequency.\n";
         help();
@@ -3012,9 +3024,8 @@ void version() {
 
 void help() {
     cerr << "ustacks " << VERSION << "\n"
-         << "ustacks -f file_path -i id -o path [-M max_dist] [-m min_cov] [-p num_threads]" << "\n"
+         << "ustacks -f file_path -o path [-M max_dist] [-m min_cov] [-p num_threads]" << "\n"
          << "  f: input file path.\n"
-         << "  i: a unique integer ID for this sample.\n"
          << "  o: output path to write results.\n"
          << "  M: Maximum distance (in nucleotides) allowed between stacks (default 2).\n"
          << "  m: Minimum depth of coverage required to create a stack (default 3).\n"
@@ -3026,11 +3037,12 @@ void help() {
          << "  H: disable calling haplotypes from secondary reads.\n"
          << "\n"
          << "  Stack assembly options:\n"
-         << "    d,--deleverage: enable the Deleveraging algorithm, used for resolving over merged tags.\n"
+         << "    --force-diff-len: allow raw input reads of different lengths, e.g. after trimming (default: ustacks perfers raw input reads of uniform length).\n"
          << "    --keep-high-cov: disable the algorithm that removes highly-repetitive stacks and nearby errors.\n"
          << "    --high-cov-thres: highly-repetitive stacks threshold, in standard deviation units (default: 3.0).\n"
          << "    --max-locus-stacks <num>: maximum number of stacks at a single de novo locus (default 3).\n"
-         << "     --k-len <len>: specify k-mer size for matching between alleles and loci (automatically calculated by default).\n\n"
+         << "     --k-len <len>: specify k-mer size for matching between alleles and loci (automatically calculated by default).\n"
+         << "    --deleverage: enable the Deleveraging algorithm, used for resolving over merged tags.\n\n"
          << "  Gapped assembly options:\n"
          << "    --max-gaps: number of gaps allowed between stacks before merging (default: 2).\n"
          << "    --min-aln-len: minimum length of aligned sequence in a gapped alignment (default: 0.80).\n\n"
